@@ -1,9 +1,8 @@
-# $Id: Piece.pm,v 1.16.2.2 2006/02/11 12:55:26 rjbs Exp $
+# $Id: Piece.pm 76 2008-03-02 20:15:09Z matt $
 
 package Time::Piece;
 
 use strict;
-use vars qw($VERSION @ISA @EXPORT %EXPORT_TAGS);
 
 require Exporter;
 require DynaLoader;
@@ -12,18 +11,18 @@ use Carp;
 use Time::Local;
 use UNIVERSAL qw(isa);
 
-@ISA = qw(Exporter DynaLoader);
+our @ISA = qw(Exporter DynaLoader);
 
-@EXPORT = qw(
+our @EXPORT = qw(
     localtime
     gmtime
 );
 
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
     ':override' => 'internal',
     );
 
-$VERSION = '1.09';
+our $VERSION = '1.13';
 
 bootstrap Time::Piece $VERSION;
 
@@ -98,10 +97,12 @@ sub parse {
 
 sub _mktime {
     my ($class, $time, $islocal) = @_;
-    $class = eval { (ref $class)->isa('Time::Piece') } ? ref $class : $class;
+    $class = eval { (ref $class) && (ref $class)->isa('Time::Piece') }
+           ? ref $class
+           : $class;
     if (ref($time)) {
-	$time->[c_epoch] = undef;
-	return wantarray ? @$time : bless [@$time, $islocal], $class;
+        $time->[c_epoch] = undef;
+        return wantarray ? @$time : bless [@$time, $islocal], $class;
     }
     _tzset();
     my @time = $islocal ?
@@ -121,6 +122,7 @@ sub export {
   for my $method (@methods) {
     if (exists $_special_exports{$method}) {
       no strict 'refs';
+      no warnings 'redefine';
       *{$to . "::$method"} = $_special_exports{$method}->($class);
     } else {
       $class->SUPER::export($to, $method);
@@ -280,6 +282,8 @@ sub isdst {
 # Thanks to Tony Olekshy <olekshy@cs.ualberta.ca> for this algorithm
 sub tzoffset {
     my $time = shift;
+    
+    return Time::Seconds->new(0) unless $time->[c_islocal];
 
     my $epoch = $time->epoch;
 
@@ -302,15 +306,15 @@ sub tzoffset {
 sub epoch {
     my $time = shift;
     if (defined($time->[c_epoch])) {
-	return $time->[c_epoch];
+        return $time->[c_epoch];
     }
     else {
-	my $epoch = $time->[c_islocal] ?
-	  timelocal(@{$time}[c_sec .. c_mon], $time->[c_year]+1900)
-	  :
-	  timegm(@{$time}[c_sec .. c_mon], $time->[c_year]+1900);
-	$time->[c_epoch] = $epoch;
-	return $epoch;
+        my $epoch = $time->[c_islocal] ?
+          timelocal(@{$time}[c_sec .. c_mon], $time->[c_year]+1900)
+          :
+          timegm(@{$time}[c_sec .. c_mon], $time->[c_year]+1900);
+        $time->[c_epoch] = $epoch;
+        return $epoch;
     }
 }
 
@@ -446,14 +450,15 @@ sub month_last_day {
 
 sub strftime {
     my $time = shift;
-    my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S %Z";
+    my $tzname = $time->[c_islocal] ? '%Z' : 'UTC';
+    my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S $tzname";
     if (!defined $time->[c_wday]) {
-	if ($time->[c_islocal]) {
+        if ($time->[c_islocal]) {
             return _strftime($format, CORE::localtime($time->epoch));
-	}
-	else {
-	    return _strftime($format, CORE::gmtime($time->epoch));
-	}
+        }
+        else {
+            return _strftime($format, CORE::gmtime($time->epoch));
+        }
     }
     return _strftime($format, (@$time)[c_sec..c_isdst]);
 }
@@ -535,7 +540,17 @@ sub subtract {
     if (UNIVERSAL::isa($rhs, 'Time::Seconds')) {
         $rhs = $rhs->seconds;
     }
-    die "Can't subtract a date from something!" if shift;
+
+    if (shift)
+    {
+	# SWAPED is set (so someone tried an expression like NOTDATE - DATE).
+	# Imitate Perl's standard behavior and return the result as if the
+	# string $time resolves to was subtracted from NOTDATE.  This way,
+	# classes which override this one and which have a stringify function
+	# that resolves to something that looks more like a number don't need
+	# to override this function.
+	return $rhs - "$time";
+    }
     
     if (UNIVERSAL::isa($rhs, 'Time::Piece')) {
         return Time::Seconds->new($time->epoch - $rhs->epoch);
@@ -576,6 +591,37 @@ sub compare {
     return $lhs <=> $rhs;
 }
 
+sub add_months {
+    my ($time, $num_months) = @_;
+    
+    croak("add_months requires a number of months") unless defined($num_months);
+    
+    my $final_month = $time->_mon + $num_months;
+    my $num_years = 0;
+    if ($final_month > 11 || $final_month < 0) {
+        # these two ops required because we have no POSIX::floor and don't
+        # want to load POSIX.pm
+        $num_years = int($final_month / 12);
+        $num_years-- if ($final_month < 0);
+        
+        $final_month = $final_month % 12;
+    }
+    
+    my $string = ($time->year + $num_years) . "-" .
+                 ($final_month + 1) . "-" .
+                 ($time->mday) . " " . $time->hms;
+    my $format = "%Y-%m-%d %H:%M:%S";
+    #warn("Parsing string: $string\n");
+    my @vals = _strptime($string, $format);
+#    warn(sprintf("got vals: %d-%d-%d %d:%d:%d\n", reverse(@vals)));
+    return scalar $time->_mktime(\@vals, $time->[c_islocal]);
+}
+
+sub add_years {
+    my ($time, $years) = @_;
+    $time->add_months($years * 12);
+}
+
 1;
 __END__
 
@@ -717,6 +763,20 @@ while examining the object will print the number of seconds (because
 of the overloading), you can also get the number of minutes, hours,
 days, weeks and years in that delta, using the Time::Seconds API.
 
+In addition to adding seconds, there are two APIs for adding months and
+years:
+
+    $t->add_months(6);
+    $t->add_years(5);
+
+The months and years can be negative for subtractions. Note that there
+is some "strange" behaviour when adding and subtracting months at the
+ends of months. Generally when the resulting month is shorter than the
+starting month then the number of overlap days is added. For example
+subtracting a month from 2008-03-31 will not result in 2008-02-31 as this
+is an impossible date. Instead you will get 2008-03-02. This appears to
+be consistent with other date manipulation tools.
+
 =head2 Date Comparisons
 
 Date comparisons are also possible, using the full suite of "<", ">",
